library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;

entity FFT_top is
	port (
				clock			: in  std_logic;
				reset			: in  std_logic;
				start			: in  std_logic;
				data_in		: in  std_logic_vector ( 15 downto 0 );
				data_out		: out std_logic_vector ( 15 downto 0 );
				read_addr	: out std_logic_vector (  9 downto 0 );
				write_addr	: out std_logic_vector (  9 downto 0 );
				wren			: out std_logic;
				FFTdone		: out std_logic
			);
end FFT_top;

architecture whole of FFT_top is

--component declaration

component FFT
	port (
		clk	: IN STD_LOGIC;
		reset_n	: IN STD_LOGIC;
		fftpts_in	: IN STD_LOGIC_VECTOR (10 DOWNTO 0);
		inverse	: IN STD_LOGIC;
		sink_valid	: IN STD_LOGIC;
		sink_sop	: IN STD_LOGIC;
		sink_eop	: IN STD_LOGIC;
		sink_real	: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		sink_imag	: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		sink_error	: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		source_ready	: IN STD_LOGIC;
		fftpts_out	: OUT STD_LOGIC_VECTOR (10 DOWNTO 0);
		sink_ready	: OUT STD_LOGIC;
		source_error	: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		source_sop	: OUT STD_LOGIC;
		source_eop	: OUT STD_LOGIC;
		source_valid	: OUT STD_LOGIC;
		source_real	: OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
		source_imag	: OUT STD_LOGIC_VECTOR (15 DOWNTO 0)
	);
end component;

component FFTinControl
	port
	(
      iReset       	: IN  STD_LOGIC;
      iStart       	: IN  STD_LOGIC;
      iClk   		 	: IN  STD_LOGIC;
		iSink_ready		: IN STD_LOGIC;
      oFFTpts 		 	: OUT STD_LOGIC_VECTOR(10 DOWNTO 0);
      oSink_real_en  : OUT STD_LOGIC;
      oSink_imag   	: OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		oRead_Addr	 	: OUT STD_LOGIC_VECTOR( 9 DOWNTO 0);
      oInverse     	: OUT STD_LOGIC;
		oSource_ready	: OUT STD_LOGIC;
		oSink_valid		: OUT STD_LOGIC;
		oSink_eop		: OUT STD_LOGIC;
		oSink_sop		: OUT STD_LOGIC;
      oSink_error  	: OUT STD_LOGIC_VECTOR(1 DOWNTO 0)
	);
end component;

component Sink_real_mux
	port
	(
      iSamp        	: IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      iSink_real_en  : IN  STD_LOGIC;
      oSink_real   	: OUT STD_LOGIC_VECTOR(15 DOWNTO 0)
	);
end component;

component FFToutControl IS
	port (
		clk				: IN STD_LOGIC;
		reset_n			: IN STD_LOGIC;
		sink_sop 		: IN STD_LOGIC;
		source_eop		: IN STD_LOGIC;
		source_valid	: IN STD_LOGIC;
		data_in			: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		data_out			: OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
		wren				: OUT STD_LOGIC;
		FFTdone			: OUT STD_LOGIC;
		write_addr		: OUT STD_LOGIC_VECTOR( 9 DOWNTO 0)
	);
end component;

component PowerGen IS
	PORT (
		source_real	: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		source_imag	: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		power_out	: OUT STD_LOGIC_VECTOR (15 DOWNTO 0)
	);
end component;
--signal declaration

signal reset_n			: STD_LOGIC;
signal sink_real_en	: STD_LOGIC;
signal inverse			: STD_LOGIC;
signal source_ready	: STD_LOGIC;
signal sink_ready		: STD_LOGIC;
signal sink_valid		: STD_LOGIC;
signal source_valid	: STD_LOGIC;
signal sink_eop		: STD_LOGIC;
signal sink_sop		: STD_LOGIC;
signal source_eop		: STD_LOGIC;
signal source_sop		: STD_LOGIC;
signal sink_error		: STD_LOGIC_VECTOR ( 1 DOWNTO 0);
signal source_error	: STD_LOGIC_VECTOR ( 1 DOWNTO 0);
signal FFTpts			: STD_LOGIC_VECTOR (10 DOWNTO 0);
signal FFTpts_out		: STD_LOGIC_VECTOR (10 DOWNTO 0);
signal sink_imag		: STD_LOGIC_VECTOR (15 DOWNTO 0);
signal sink_real		: STD_LOGIC_VECTOR (15 DOWNTO 0);
signal source_imag	: STD_LOGIC_VECTOR (15 DOWNTO 0);
signal source_real	: STD_LOGIC_VECTOR (15 DOWNTO 0);
signal power_out		: STD_LOGIC_VECTOR (15 DOWNTO 0);
--component connection
begin
reset_n <= not reset;
inControl: FFTinControl port map ( reset, start, clock, sink_ready, FFTpts, sink_real_en, sink_imag, read_addr, inverse, source_ready, sink_valid, sink_eop, sink_sop, sink_error);
Real_control: sink_real_mux port map( data_in, sink_real_en, sink_real);
FFTcore: FFT port map ( clock, reset_n, FFTpts, inverse, sink_valid, sink_sop, sink_eop, sink_real, sink_imag, sink_error, source_ready, FFTpts_out, sink_ready, source_error, source_sop, source_eop, source_valid, source_real, source_imag);
PowerGenerator: PowerGen port map ( source_real, source_imag, power_out);
outControl: FFToutControl port map ( clock, reset_n, sink_sop, source_eop, source_valid, power_out, data_out, wren, FFTdone, write_addr);

end whole;
