LIBRARY ieee;
   USE ieee.std_logic_1164.all;
   USE ieee.std_logic_unsigned.all;

entity FFTinControl is

	port

	(

      iReset       	: IN  STD_LOGIC;
      iStart       	: IN  STD_LOGIC;
      iClk   		 	: IN  STD_LOGIC;
      iSink_ready		: IN STD_LOGIC;
      oFFTpts 		 	: OUT STD_LOGIC_VECTOR(10 DOWNTO 0);
      oSink_real_en  : OUT STD_LOGIC;
      oSink_imag   	: OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		oRead_Addr	 	: OUT STD_LOGIC_VECTOR( 9 DOWNTO 0);
      oInverse     	: OUT STD_LOGIC;
		oSource_ready	: OUT STD_LOGIC;
		oSink_valid		: OUT STD_LOGIC;
		oSink_eop		: OUT STD_LOGIC;
		oSink_sop		: OUT STD_LOGIC;
      oSink_error  	: OUT STD_LOGIC_VECTOR(1 DOWNTO 0)

	);

end entity;



architecture rtl of FFTinControl is

	-- Build an enumerated type for the state machine
	type state_type is (INSOP, INING1, INING2, INEOP, INWAIT);
	-- Register to hold the current state
	signal instate : state_type;
	signal incount : std_LOGIC_VECTOR ( 9 downto 0);
	
begin

   oSource_ready <= '1';
   oSink_error <= "00";
   oSink_imag <= (others => '0');
   oRead_Addr <= incount;	
	oFFTpts <= "10000000000";
	oInverse <= '0';
	
	process (iClk, iReset)
	begin
		if iReset = '1' then
            oSink_real_en <= '0';
            oSink_valid <= '0';
            incount <= (others =>'0');
            instate <= INWAIT;
				
		elsif (rising_edge(iClk)) then
			case instate is
			
				WHEN INSOP =>
                  oSink_sop <= '1';
						oSink_eop <= '0';
                  oSink_valid <= '1';
                  oSink_real_en <= '1';
                  incount(0) <= '1';
						incount(9 downto 1) <=(others =>'0');
						if (iSink_ready = '1') then
                  instate <= INING1;
						else 
						instate <= INING2;
						end if;
               
               WHEN INING1 =>
						oSink_sop <= '0';
                  oSink_eop <= '0';
                  oSink_real_en <= '1';

                  IF (incount = (1024 - 2)) THEN
                     instate <= INEOP;
                  ELSE
						IF (iSink_ready = '1') THEN
                     incount <= incount + '1';
                     instate <= INING1;
						ELSE
							incount <= incount;
							instate <= INING2;
                  END IF;
                  END IF;

               WHEN INING2 =>
						oSink_sop <= '0';
                  oSink_eop <= '0';
                  oSink_real_en <= '1';
                  
                  IF (incount = (1024 - 2)) THEN
                     instate <= INEOP;
                  ELSE
						IF (iSink_ready = '1') THEN
                     incount <= incount + '1';
                     instate <= INING1;
						ELSE
                  incount <= incount;
							instate <= INING2;
                  END IF;	
                  END IF;
               
               WHEN INEOP =>
						oSink_sop <= '0';
                  oSink_eop <= '1';
                  oSink_real_en <= '1';
                  instate <= INWAIT;
               
               WHEN INWAIT =>
                  oSink_real_en <= '0';
						oSink_sop <= '0';
                  oSink_eop <= '0';
                  oSink_valid <= '0';
						incount <= (others =>'0');
                  IF (iStart = '1') AND (iSink_ready = '1') THEN
                     instate <= INSOP;
                  ELSE
                     instate <= INWAIT;
                  END IF;
						
					WHEN others =>
                  oSink_real_en <= '0';
                  oSink_eop <= '0';
                  oSink_valid <= '0';
						incount <= (others =>'0');
                  IF (iStart = '1') THEN
                     instate <= INSOP;
                  ELSE
                     instate <= INWAIT;
                  END IF;
						
			end case;
		end if;
	end process;
end rtl;
